"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.outFile = outFile;
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const signale_1 = __importDefault(require("signale"));
const code_1 = require("./code");
const types_1 = require("./types");
/**
 * check should overwrite file
 */
function checkToOverwrite(writeFileMode, outFileName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const isExisted = yield fs_extra_1.default.pathExists(outFileName);
            if (!isExisted) {
                return true;
            }
            const existedTip = `${outFileName} is already existed`;
            if (writeFileMode === types_1.WriteFileMode.error) {
                throw Error(chalk_1.default.redBright(existedTip));
            }
            if (writeFileMode === types_1.WriteFileMode.warn) {
                signale_1.default.warn(chalk_1.default.yellowBright(existedTip));
                return false;
            }
            if (writeFileMode === types_1.WriteFileMode.overwrite) {
                return true;
            }
            return false;
        }
        catch (error) {
            return false;
        }
    });
}
/**
 * output file
 */
function outFile(_a) {
    return __awaiter(this, arguments, void 0, function* ({ tips, outFileName, code, writeFileMode = types_1.WriteFileMode.skip }) {
        const { start, success, error } = tips;
        const allowed = yield checkToOverwrite(writeFileMode, outFileName);
        if (!allowed)
            return;
        signale_1.default.start(chalk_1.default.greenBright(start));
        const normalizedCode = yield (0, code_1.formatCode)({
            source: code,
            filePath: outFileName,
        });
        try {
            yield fs_extra_1.default.outputFile(outFileName, normalizedCode);
            signale_1.default.success(chalk_1.default.greenBright(success));
        }
        catch (e) {
            signale_1.default.error(chalk_1.default.redBright(error));
            signale_1.default.error(chalk_1.default.redBright(e));
        }
    });
}
