"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRenderer = createRenderer;
exports.recursiveRemoveFieldFromSchema = recursiveRemoveFieldFromSchema;
const generator_1 = __importDefault(require("@babel/generator"));
const template_1 = __importDefault(require("@babel/template"));
const t = __importStar(require("@babel/types"));
const camelcase_1 = __importDefault(require("camelcase"));
const case_1 = require("./case");
const constants_1 = require("./constants");
const identifier_1 = require("./identifier");
const service_1 = require("./service");
const template_2 = require("./template");
function isSupportParametersType(type) {
    return constants_1.SUPPORT_PARAMETERS_TYPES.includes(type);
}
function formatAPIFunctionName(method, url, parameters) {
    const pathParameters = parameters.filter((parameter) => parameter.in === 'path').map((parameter) => parameter.name);
    const suffix = pathParameters.length ? 'By' + pathParameters.map((name) => (0, case_1.pascalCase)(name)).join('And') : '';
    const apiName = url
        .split('/')
        .filter((name) => !pathParameters.map((parameter) => `{${parameter}}`).includes(name))
        .map((str) => (0, case_1.pascalCase)((0, identifier_1.formatIdentifierName)(str)))
        .join('');
    return (0, camelcase_1.default)(method.toLowerCase() + apiName + suffix);
}
function formatParameters(parameters = []) {
    return parameters.map((it) => (Object.assign(Object.assign({}, it), { type: it.in })));
}
function getResponseRef(response) {
    var _a;
    if (!response) {
        return '';
    }
    if ('content' in response) {
        // open api v3
        const responseContent = response.content;
        const jsonResponse = (_a = responseContent === null || responseContent === void 0 ? void 0 : responseContent['application/json']) !== null && _a !== void 0 ? _a : responseContent === null || responseContent === void 0 ? void 0 : responseContent['*/*'];
        if (jsonResponse) {
            const responseSchema = jsonResponse.schema;
            if (responseSchema === null || responseSchema === void 0 ? void 0 : responseSchema.$ref) {
                return responseSchema.$ref;
            }
        }
    }
    else if ('schema' in response) {
        // open api v2
        if (response.schema && response.schema.type !== 'file' && response.schema.$ref) {
            return response.schema.$ref;
        }
    }
    return '';
}
function formatResponse(response) {
    var _a;
    if (!response) {
        return undefined;
    }
    const ref = getResponseRef(response);
    let rawType = '';
    let items;
    if ('content' in response) {
        // open api v3
        const responseContent = response.content;
        const jsonResponse = (_a = responseContent === null || responseContent === void 0 ? void 0 : responseContent['application/json']) !== null && _a !== void 0 ? _a : responseContent === null || responseContent === void 0 ? void 0 : responseContent['*/*'];
        if (jsonResponse) {
            const responseSchema = jsonResponse.schema;
            if (responseSchema && 'type' in responseSchema) {
                rawType = responseSchema.type;
                if (rawType === 'array') {
                    items = responseSchema.items;
                }
            }
        }
    }
    else if ('schema' in response) {
        // open api v2
        if (response.schema) {
            if (response.schema.type === 'file') {
                return 'Blob';
            }
            else if (response.schema.type) {
                rawType = response.schema.type;
                if (rawType === 'array') {
                    items = response.schema.items;
                }
            }
        }
    }
    if (ref) {
        const type = ref.split('/').pop() || '';
        return (0, identifier_1.formatTypeName)(type);
    }
    else if (rawType) {
        if (rawType === 'array') {
            if (items === null || items === void 0 ? void 0 : items.$ref) {
                const type = items.$ref.split('/').pop() || '';
                return (0, identifier_1.formatTypeName)(type) + '[]';
            }
            else if (items === null || items === void 0 ? void 0 : items.type) {
                if ((items === null || items === void 0 ? void 0 : items.type) && /^[a-z]/.test(items === null || items === void 0 ? void 0 : items.type)) {
                    if ((items === null || items === void 0 ? void 0 : items.type) === 'object') {
                        return 'Record<string, any>[]';
                    }
                    // other basic value type
                    return items.type + '[]';
                }
                return (0, identifier_1.formatTypeName)(items.type) + '[]';
            }
        }
        return rawType;
    }
    else {
        return undefined;
    }
}
function formatOperationId(operationId = '') {
    return (0, identifier_1.formatTypeName)(operationId);
}
function replaceParams(url, locator) {
    return url.replace(/\{(\w+)\}/g, `\${${locator}.$1}`);
}
function createResponse(options) {
    const { response, service, isV3 = false, extractField, raw, schema } = options;
    const serviceNamespace = (0, case_1.pascalCase)(service);
    if (response) {
        if (response && /^[a-z]/.test(response)) {
            if (response === 'object') {
                return t.identifier('Record<string, any>');
            }
            // other basic value type
            return t.identifier(response);
        }
        else if (response.startsWith('Record<string, any>')) {
            return t.identifier(response);
        }
        else if (response.endsWith('Void')) {
            return t.identifier('void');
        }
        else if (extractField) {
            // need extract field
            if (schema && raw) {
                const ref = getResponseRef(raw);
                // if extract field not exist in response, remove it
                if (ref) {
                    const fields = recursiveRemoveFieldFromSchema(Array.isArray(extractField) ? extractField : [extractField], ref, schema, 0);
                    const identifier = fields.reduce((prev, cur) => {
                        return `Required<${prev}>['${cur}']`;
                    }, `${serviceNamespace}${isV3 ? '.Schemas' : ''}.${response}`);
                    return t.identifier(identifier);
                }
            }
        }
        return t.identifier(`${serviceNamespace}${isV3 ? '.Schemas' : ''}.${response}`);
    }
    else {
        return t.identifier('void');
    }
}
function addComments(node, comment) {
    const value = comment.map((str) => `*\n* ${str}`).join('\n') + '\n';
    return t.addComments(node, 'leading', [
        {
            type: 'CommentBlock',
            value,
        },
    ]);
}
function pickParams(parameters) {
    return function (type) {
        return parameters.filter((it) => it.type === type);
    };
}
function createParams(parameters, service, operationId, configTypeName = 'AxiosRequestConfig') {
    const serviceNamespace = (0, case_1.pascalCase)(service);
    const types = [...new Set(parameters.map((it) => it.type))].sort((a, b) => {
        return constants_1.SUPPORT_PARAMETERS_TYPES.indexOf(a) - constants_1.SUPPORT_PARAMETERS_TYPES.indexOf(b);
    });
    const bodyParameters = parameters.filter((it) => it.type === 'body');
    const configParamNode = t.identifier('config');
    configParamNode.typeAnnotation = t.tsTypeAnnotation(t.tsTypeReference(t.identifier(configTypeName)));
    configParamNode.optional = true;
    return types
        .filter(isSupportParametersType)
        .map((type) => {
        if (type === 'body') {
            return paramsTypeAst('bodyParameters')(serviceNamespace, operationId, bodyParameters.length === 1 ? bodyParameters[0].name : '');
        }
        else if (type === 'formData') {
            return paramsTypeAst('formDataParameters')(serviceNamespace, operationId);
        }
        else if (type === 'path') {
            return paramsTypeAst('pathParameters')(serviceNamespace, operationId);
        }
        else {
            return paramsTypeAst('queryParameters')(serviceNamespace, operationId);
        }
    })
        .concat(configParamNode);
}
function createCalleeArgs(url, method, parameters) {
    const pick = pickParams(parameters);
    const queryParams = pick('query');
    const pathParams = pick('path');
    const bodyParams = pick('body');
    const formDataParams = pick('formData');
    if (pathParams.length !== 0) {
        url = replaceParams(url, 'pathParameters');
    }
    let configStr = `
  {
    url: ${url.includes('${') ? '`' + url + '`' : `'${url}'`},
    method: '${method}',
  `;
    if (bodyParams.length) {
        configStr += 'data: bodyParameters, \n';
    }
    else if (formDataParams.length) {
        configStr += 'data: formDataParameters, \n';
    }
    if (queryParams.length) {
        configStr += 'params: queryParameters, \n';
    }
    configStr += `...config,
  }`;
    return configStr;
}
function paramsTypeAst(name) {
    const node = t.identifier(name);
    const type = (node.typeAnnotation = t.typeAnnotation(t.anyTypeAnnotation()));
    if (name === 'bodyParameters') {
        return function (serviceNamespace, operationId, parameterName) {
            let parameterType = `${serviceNamespace}.${(0, identifier_1.formatTypeName)(operationId)}.${(0, identifier_1.formatTypeName)(name)}`;
            if (parameterName) {
                parameterType += `['${parameterName}']`;
            }
            type.typeAnnotation = t.tsTypeReference(t.identifier(parameterType));
            return node;
        };
    }
    else if (name === 'formDataParameters') {
        return function () {
            type.typeAnnotation = t.tsTypeReference(t.identifier('FormData'));
            return node;
        };
    }
    else {
        return function (serviceNamespace, operationId) {
            type.typeAnnotation = t.tsTypeReference(t.identifier(`${serviceNamespace}.${(0, identifier_1.formatTypeName)(operationId)}.${(0, identifier_1.formatTypeName)(name)}`));
            return node;
        };
    }
}
function createItem(api, options = {}) {
    const { method, uri, description = '', summary = description, parameters = [], operationId = '', successResponse, } = api;
    const { urlFormatter = (url) => url, nameFormatter = formatAPIFunctionName } = options;
    return {
        method: method.toString(),
        url: urlFormatter(uri),
        summary,
        name: nameFormatter(method, uri, parameters, formatAPIFunctionName),
        parameters: formatParameters(parameters),
        response: formatResponse(successResponse),
        operationId: formatOperationId(operationId),
        rawResponse: successResponse,
    };
}
function createRenderer(templateSource, options) {
    const templateAst = (0, template_1.default)(templateSource, {
        plugins: ['typescript'],
    });
    const templatePlaceholders = (0, template_2.getTemplateParams)(templateSource);
    return (api, service) => {
        var _a, _b;
        const { name, method, url, summary, parameters = [], operationId = '', response, rawResponse, } = createItem(api, options);
        const allArgs = {
            NAME: name,
            SERVICE: (0, service_1.formatService)(service),
            RESPONSE: createResponse({
                response,
                operationId,
                service,
                isV3: (options === null || options === void 0 ? void 0 : options.openApiVersion) === 3,
                extractField: options === null || options === void 0 ? void 0 : options.extractField,
                schema: options === null || options === void 0 ? void 0 : options.schema,
                raw: rawResponse,
            }),
            PARAMS: createParams(parameters, service, operationId, options === null || options === void 0 ? void 0 : options.configParamTypeName),
            ARGS: createCalleeArgs(url, method, parameters),
        };
        const neededArgs = templatePlaceholders.reduce((acc, cur) => {
            acc[cur] = allArgs[cur];
            return acc;
        }, {});
        const astNode = templateAst(neededArgs);
        const withCommentsNode = addComments(astNode, [
            summary,
            api.description && api.description !== summary ? `@description ${api.description}` : '',
            api.deprecated ? '@deprecated' : '',
            ((_a = api.externalDocs) === null || _a === void 0 ? void 0 : _a.url) ? `@see ${(_b = api.externalDocs) === null || _b === void 0 ? void 0 : _b.url}` : '',
        ].filter(Boolean));
        const { code } = (0, generator_1.default)(withCommentsNode);
        return code;
    };
}
function recursiveRemoveFieldFromSchema(fields, refStr, schema, index = 0) {
    var _a, _b, _c;
    const ref = refStr.split('/');
    const field = fields[index];
    const definitionName = ref.pop();
    if (typeof schema.content === 'object') {
        const definitions = (_c = (_a = schema.content.definitions) !== null && _a !== void 0 ? _a : (_b = schema.content.components) === null || _b === void 0 ? void 0 : _b.schemas) !== null && _c !== void 0 ? _c : {};
        const responseSchema = definitions[definitionName];
        if (responseSchema && typeof responseSchema === 'object') {
            const { properties } = responseSchema;
            if (properties) {
                const property = properties[field];
                if (!property) {
                    // if field not exist, remove all fields
                    return fields.filter((_, i) => i < index);
                }
                else if (typeof property === 'object' && property.$ref) {
                    return recursiveRemoveFieldFromSchema(fields, property.$ref.split('/').pop(), schema, index + 1);
                }
            }
        }
    }
    return [...fields];
}
